/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains all the functions necessary for catalog handling.
 */
var Catalog = function() {
	var config = {
		XHTML_NS : "http://www.w3.org/1999/xhtml",
		CATHtml : [],
		CATCss : [],
		CATJs : [],
		rafStatus : [],
		expiredBooks : [],
		cdpRef : [],
		expired: false,
		displayedCatalogExpiry: false	//	To display catalog expiry dialog only one time, when the application is launched.
	};
	
	/**
	 * Unloads the catalog by clearing the variables.
	 */
	var unload = function() {
		config.CATHtml.splice(0, config.CATHtml.length);
		config.CATCss.splice(0, config.CATCss.length);
		config.CATJs.splice(0, config.CATJs.length);
		config.rafStatus.splice(0, config.rafStatus.length);
		config.expiredBooks.splice(0, config.expiredBooks.length);
		config.cdpRef.splice(0, config.cdpRef.length);
		config.expired = false;
		config.displayedCatalogExpiry = false;
	};

	/**
	 * Load the requested catalog
	 * @params catalogId The Id of the catalog to load.
	 */
	var load = function(catalogId) {
		var self = this;
		
		//Unload the previous catalog if any.
		self.unload();
		
		//Reset the books current position.
		CdpNav.book.curr_pos = -1;
		//If book downloading is in progress, then show the status bar.
		if(CdpDownload.config.isDownloading){
			AME.showStatusBar();
		}
		
		//Get the catalog preferences and set it.
		var catPref = JSON.parse(self.getPreferences(catalogId));
		if(typeof catPref.open_width !== 'undefined' && typeof catPref.open_height !== 'undefined') {
			window.resizeTo(catPref.open_width, catPref.open_height);
		}
		
		//Get catalog data.
		self.getHtmlData(catalogId);
		self.getCssData(catalogId);
		self.getJsData(catalogId);

		$('#doc-viewer-azd').bind('DOMContentLoaded', function() {
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
			//Set catalog data once "azardi2_default_catalog_doc.html" is loaded.
			self.setCssData();
			self.setJsData();
			self.setHtmlData();
			
			//Set the catalog expiry for catalogs other than MASTERCAT
			if(catalogId !== 'MASTERCAT') {
				self.setCatalogExpiry(catalogId);
				self.getButtonStatus();
				//The following function is in the js file that comes with the catalog.
				document.getElementById('doc-viewer-azd').contentWindow.initCatalog();
				self.setButtonStatus();
			}
		});
		$('#doc-viewer-azd').attr('src', '');
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_catalog_doc.html');
	};
	
	/**
	 * Get the catalog preferences from the database.
	 * @params catId Id of the catalog.
	 * @returns {object} Catalog preferences in key value pair.
	 */
	var getPreferences = function(catId) {
		var self = this;
		
		//Check if the db file exists.
		var fileObj = getFileObject(AME.getCdpDbFilePath());
		if(!fileObj.exists()) {
			return;
		}
		//Get the preferences from the database.
		var values = ['value'];
		var tableName = 'CAT_Info';
		var conditions = new Array(['cat_id', catId, 'TEXT'],
								   ['key', 'cat_pref', 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, tableName, conditions);
		var catPref = {};
		
		while(statement.executeStep()) {
			catPref = statement.row.value;
		}
		statement.finalize();
		
		return catPref;
	};
	
	/**
	 * Initilizes the catalog expiry loop
	 * @params catalogId Id of the catalog
	 */
	var setCatalogExpiry = function(catalogId) {
		var self = this;
		
		//Get the list of cdp in this catalog.
		self.getAllCdp(catalogId);
		//Get the messages.
		var renew_catalog_website = document.getElementById('doc-viewer-azd').contentWindow.renew_catalog_website;
		var renew_cat_url = document.getElementById('doc-viewer-azd').contentWindow.renew_cat_url;
		var renew_catalog_expiring_msg = document.getElementById('doc-viewer-azd').contentWindow.renew_catalog_expiring_msg;
		var renew_catalog_expired_msg = document.getElementById('doc-viewer-azd').contentWindow.renew_catalog_expired_msg;

		// Get the catalog expiry date from database
		var catalogExpiryDate = self.getCatalogExpiryDate(catalogId);
		
		if(!config.displayedCatalogExpiry) {
			// If the catalog expiry dialog is not shown then show it, if catalog has expired
			
			config.displayedCatalogExpiry = true;
			
			if(catalogExpiryDate == 'never') {
				// If the catalog expiry is "never" then just check the book expiry.
				self.setBookExpiry();
				return;
			}
			
			// Check if catalog is expired checking the dates
			if(self.isCatalogExpired(catalogExpiryDate)) {
				// Write cat expired in database
				self.writeCatStatus(catalogId, 'expired');
				// Remove all books from database
				self.removeAllBooks();
				// Set expired status on all books
				self.expireAllBooks();
			}
	
			// Check if catalog is expired (from database)
			if(self.isCatalogAlreadyExpired(catalogId)) {
				config.expired = true;
				// if catalog has expired
				// show renewal dialog
				var renewed = self.showCatalogRenewalDialog(catalogId, renew_catalog_website, renew_catalog_expired_msg, renew_cat_url);
				if(renewed) {
					// change catalog status to active
					self.writeCatStatus(catalogId, 'active');
					// change status of all books
					//Just removing the book status from database makes them downloadable again.
					for(var i = 0; i < config.cdpRef.length; i++) {
						var fileObj = getFileObject(AME.getCdpDbFilePath());
						var table_name = 'RAF_Master';
						var conditions = new Array(['field_id', config.cdpRef[i], 'TEXT']);
						deleteRowFromDatabase(fileObj,table_name, conditions);
					};
				}
			} else {
				// if catalog has not expired
				// check if catalog is expiring within next few days
				if(self.isCatalogExpiring(catalogExpiryDate)) {
					// if catalog is expiring within 7 days.
					var daysRemaining = self.expiryDaysRemaining(catalogExpiryDate);
					var snippet;
					if(daysRemaining == 1) {
						snippet = 'today';
					}
					else{
						snippet = 'in '+daysRemaining+' days';
					}
					
					// show renewal dialog
					var renewed = self.showCatalogRenewalDialog(catalogId, renew_catalog_website, renew_catalog_expiring_msg, renew_cat_url, snippet);
				}
				// check if books are expired
				self.setBookExpiry();
			}
		}
	};
	
	/**
	 * Expires all the books
	 */
	var expireAllBooks = function() {
		var self = this;

		for(var i = 0; i < config.cdpRef.length; i++) {
			self.writeBookStatus(config.cdpRef[i], 'expired');
		}
	};
	
	/**
	 * Shows renewal dialog for catalog
	 * @params catalogId Id of the catalog
	 * @params renew_catalog_website Http address of the website where user will get the code.
	 * @params renew_catalog_message Message to display to user
	 * @params snippet extra bit of generated message
	 * @returns {boolean} true if user renews catalog, false otherwise
	 */
	var showCatalogRenewalDialog = function(catalogId, renew_catalog_website, renew_catalog_msg, renew_cat_url, snippet) {
		var self = this;

		var readerToken = AME.getReaderToken();
		var params = {
			inn : {
				type : 'renew_catalog',
				catId : catalogId,
				readerToken : readerToken,
				renewWebsite: renew_catalog_website,
				renewMsg : renew_catalog_msg,
				renewCatUrl : renew_cat_url,
				snippet: snippet
			},
			out : {
				out : null
			}
		};
		window.openDialog("chrome://azardi2/content/azd_book_renew.xul", "Subscription Expired", "chrome,width=600,height=260, dialog, modal, resizable=no, dependent, centerscreen", params).focus();

		if(!params.out.out) {
			// Catalog is not renewed
			return false;
		} else {
			// Catalog is renewed
			return true;
		}
	};
	
	/**
	 * Checks if the catalog is alreay expired from database
	 * @params catalogId Id of the catalog.
	 * @returns {boolean} true is catalog is expired, false otherwise.
	 */
	var isCatalogAlreadyExpired = function(catalogId) {
		var self = this;

		var fileObj = getFileObject(AME.getCdpDbFilePath());
		var table_name = 'RAF_Master';

		var values = ['raf_value'];
		var conditions = new Array(['raf_key', 'status', 'TEXT'],
								   ['field_id', catalogId, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		var raf_value;
		while(statement.executeStep()) {
			raf_value = statement.row.raf_value;
		}

		if(raf_value == 'expired') {
			return true;
		}
		else {
			return false;
		}
	};
	
	/**
	 * Get expiry date of the catalog from database.
	 * @params catalogId Id of the catalog.
	 * @returns {string} Expiry date
	 */
	var getCatalogExpiryDate = function(catalogId) {
		var self = this;

		var fileObj = getFileObject(AME.getCdpDbFilePath());
		var table_name = 'RAF_Master';
		var values = ['raf_value'];
		var conditions = new Array(['raf_key', 'expiry_date', 'TEXT'],
								   ['field_id', catalogId, 'TEXT']);

		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		var raf_value;
		while(statement.executeStep()) {
			raf_value = statement.row.raf_value;
		}
		//Decrypt the expiry date using catalog id as a key.
		raf_value = GibberishAES.dec(raf_value, catalogId);
		
		return raf_value;
	};
	
	/**
	 * Checks if the catalog is going to expire in next few days.
	 * @params expiryDate Expiry date of the catalog
	 * @returns {boolean} true if catalog is going to expire, false otherwise.
	 */
	var isCatalogExpiring = function(expiryDate) {
		var self = this;

		var eD = new Date();
		var eMonth =  expiryDate.substr(4, 2) - 1;
		eD.setUTCFullYear(expiryDate.substr(0, 4), eMonth, expiryDate.substr(6, 2));
		eD.setUTCHours(expiryDate.substr(8, 2), expiryDate.substr(10, 2), expiryDate.substr(12, 2), expiryDate.substr(14, 3));

		var tD = new Date();
		var timeRemaining = eD - tD;
		var sevenDays = 1000 * 60 * 60 * 24 * 7;
		if(timeRemaining < sevenDays) {
			return true;
		}
		else {
			return false;
		}
	};
	
	/**
	 * Calculates the number of days remaining before catalog expires.
	 * @params expiryDate Expiry date of the catalog.
	 * @returns {string} number of days remaining for catalog expiry
	 */
	var expiryDaysRemaining = function (expiryDate) {
		var self = this;
		
		var eD = new Date();
		var eMonth =  expiryDate.substr(4, 2) - 1;
		eD.setUTCFullYear(expiryDate.substr(0, 4), eMonth, expiryDate.substr(6, 2));
		eD.setUTCHours(expiryDate.substr(8, 2), expiryDate.substr(10, 2), expiryDate.substr(12, 2), expiryDate.substr(14, 3));
		
		var tD = new Date();
		var timeRemaining = eD - tD;
		
		daysRemaining = Math.ceil(timeRemaining / (1000*60*60*24));
		return daysRemaining;
	};
	
	/**
	 * Check if the catalog is expired.
	 * @params expireyDate Expiry date of the catalog
	 * @returns {boolean} true if the catalog is expired, false otherwise
	 */
	var isCatalogExpired = function(expiryDate) {
		var self = this;

		var eD = new Date();
		var eMonth =  expiryDate.substr(4, 2) - 1;
		eD.setUTCFullYear(expiryDate.substr(0, 4), eMonth, expiryDate.substr(6, 2));
		eD.setUTCHours(expiryDate.substr(8, 2), expiryDate.substr(10, 2), expiryDate.substr(12, 2), expiryDate.substr(14, 3));
		var tD = new Date();
		
		if(eD >= tD) {
			return false;
		} 
		else {
			return true;
		}
	};
	
	/**
	 * Writes the status of the catalog in database.
	 * @params catalogId Id of the catalog.
	 * @params staus Current status of the catalog expired|active
	 */
	var writeCatStatus = function(catalogId, status) {
		var self = this;

		var fileObj = getFileObject(AME.getCdpDbFilePath());
		var table_name = 'RAF_Master';
		var values = ['raf_value'];
		var conditions = new Array(['field_id', catalogId, 'TEXT'],
								   ['raf_key', 'status', 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		while(statement.executeStep()) {
			var raf_value = statement.row.raf_value;
		}
		values = new Array(['field_id', catalogId, 'TEXT'],
						   ['raf_key', 'status', 'TEXT'],
						   ['raf_value', status, 'TEXT'],
						   ['type', 'CAT', 'TEXT']);

		if( typeof raf_value == 'undefined') {
			//Store value for first time.
			insertRowIntoDatabase(fileObj, values, table_name);
		} else {
			//Update the existing value.
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
		}
	};
	
	/**
	 * Get the button status of each book in catalog.
	 */
	var getButtonStatus = function() {
		var self = this;

		var fileObj = getFileObject(AME.getCdpDbFilePath());
		var table_name = 'RAF_Master';
		var values = ['field_id', 'raf_value'];
		var conditions = new Array(['raf_key', 'status', 'TEXT'],
								   ['type', 'CDP', 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		var rafStatus = new Array();
		while(statement.executeStep()) {
			var data = new Array();
			data.field_id = statement.row.field_id;
			data.raf_value = statement.row.raf_value;
			config.rafStatus.push(data);
		}
	};
	
	/**
	 * Sets the button status of each book of catalog.
	 */
	var setButtonStatus = function() {
		var self = this;
		
		//Following functions are in the js file that comes with the catalog.
		for(var i = 0; i < config.rafStatus.length; i++) {
			if(config.rafStatus[i].raf_value == 'downloaded') {
				document.getElementById('doc-viewer-azd').contentWindow.hideDownloadButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.hideRemoveButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.hideReadButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.hideExpireButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.showInstallButton(config.rafStatus[i].field_id);
				document.getElementById("doc-viewer-azd").contentWindow.disableCoverRead(config.rafStatus[i].field_id);
			} else if(config.rafStatus[i].raf_value == 'installed') {
				document.getElementById('doc-viewer-azd').contentWindow.hideDownloadButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.hideInstallButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.hideExpireButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.showRemoveButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.showReadButton(config.rafStatus[i].field_id);
				document.getElementById("doc-viewer-azd").contentWindow.enableCoverRead(config.rafStatus[i].field_id);
			} else if(config.rafStatus[i].raf_value == 'expired') {
				document.getElementById('doc-viewer-azd').contentWindow.hideDownloadButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.hideInstallButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.hideRemoveButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.hideReadButton(config.rafStatus[i].field_id);
				document.getElementById('doc-viewer-azd').contentWindow.showExpireButton(config.rafStatus[i].field_id);
				document.getElementById("doc-viewer-azd").contentWindow.disableCoverRead(config.rafStatus[i].field_id);
			}
		};
		
		if(CdpDownload.config.cdpId != ''){
			document.getElementById("doc-viewer-azd").contentWindow.disableDownloadButton(CdpDownload.config.cdpId);
		}
	};
	
	/**
	 * Get the html content of catalog
	 * @params catalogId Id of the catalog
	 */
	var getHtmlData = function(catalogId) {
		var self = this;
		
		//Check if the database file exists
		var fileObj = getFileObject(AME.getCdpDbFilePath());
		if(!fileObj.exists()) {
			return;
		}
		//Get html content from the database.
		var values = ['id', 'cat_id', 'title', 'description', 'publisher', 'cat_content', 'cat_info'];
		var tableName = 'CAT_Master';
		var conditions = new Array(['cat_id', catalogId, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, tableName, conditions);

		while(statement.executeStep()) {
			var data = new Array();
			data['id'] = statement.row.id;
			data['cat_id'] = statement.row.cat_id;
			data['title'] = statement.row.title;
			data['description'] = statement.row.description;
			data['publisher'] = statement.row.publisher;
			data['cat_content'] = statement.row.cat_content;
			data['cat_info'] = statement.row.cat_info;
			config.CATHtml.push(data);
		}
		statement.finalize();
	}
	
	/**
	 * Get the css content of catalog
	 * @params catalogId Id of the catalog
	 */
	var getCssData = function(catalogId) {
		var self = this;
		
		//Check if the database file exists.
		var fileObj = getFileObject(AME.getCdpDbFilePath());
		if(!fileObj.exists()) {
			return;
		}
		//Get css content from the database.
		var values = ['id', 'cat_id', 'filename', 'file_content'];
		var tableName = 'CAT_Presentation_css';
		var conditions = new Array(['cat_id', catalogId, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, tableName, conditions);

		while(statement.executeStep()) {
			var data = new Array();
			data['id'] = statement.row.id;
			data['cat_id'] = statement.row.cat_id;
			data['filename'] = statement.row.filename;
			data['file_content'] = statement.row.file_content;
			config.CATCss.push(data);
		}
		statement.finalize();
	}
	
	/**
	 * Get the js content of catalog
	 * @params catalogId Id of the catalog
	 */
	var getJsData = function(catalogId) {
		var self = this;
		
		//Check if the database file exists.
		var fileObj = getFileObject(AME.getCdpDbFilePath());
		if(!fileObj.exists()) {
			return;
		}
		//Get js content from the database.
		var values = ['id', 'cat_id', 'filename', 'file_content'];
		var tableName = 'CAT_Presentation_js';
		var conditions = new Array(['cat_id', catalogId, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, tableName, conditions);

		while(statement.executeStep()) {
			var data = new Array();
			data['id'] = statement.row.id;
			data['cat_id'] = statement.row.cat_id;
			data['filename'] = statement.row.filename;
			data['file_content'] = statement.row.file_content;
			config.CATJs.push(data);
		}
		statement.finalize();
	}
	
	/**
	 * Set the html content of catalog.
	 */
	var setHtmlData = function() {
		$('#doc-viewer-azd').contents().find('body').html(config.CATHtml[0]['cat_content']);
	};
	
	/**
	 * Set the css content of catalog.
	 */
	var setCssData = function() {
		var self = this;

		for(var i = 0; i < config.CATCss.length; i++) {
			var styleTag = '<style type="text/css">' + config.CATCss[i]['file_content'] + '</style>';
			$('#doc-viewer-azd').contents().find('head').append(styleTag);
		};
	};
	
	/**
	 * Set the js content of catalog.
	 */
	var setJsData = function() {
		var self = this;

		for(var i = 0; i < config.CATJs.length; i++) {
			var scriptTag = document.createElementNS(config.XHTML_NS, 'script');
			var scriptContent = document.createTextNode(config.CATJs[i]['file_content']);
			scriptTag.appendChild(scriptContent);
			document.getElementById('doc-viewer-azd').contentWindow.document.getElementsByTagName('head')[0].appendChild(scriptTag);
		}
	};
	
	var installBook = function(cdpId, fullTextSearch) {
		var self = this;

		var tempDownloadPath = getFileObject(AME.getTempDownloadPath());
		tempDownloadPath.append(cdpId + '.azd');
		var params = {
			inn : {
				tempDownloadPath : tempDownloadPath.path,
				cdpId : cdpId,
				fullTextSearch: fullTextSearch,
				type : 'book_install'
			},
			out : null
		};
		window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=300, height=200, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();
		if(params.out) {
			// self.writeBookStatus(cdpId, 'installed');
			AME.showMessage('The book installation is finished. Click the "READ" button to read the book.', 'info', true, null);
			document.getElementById("doc-viewer-azd").contentWindow.hideInstallButton(cdpId);
			document.getElementById("doc-viewer-azd").contentWindow.showRemoveButton(cdpId);
			document.getElementById("doc-viewer-azd").contentWindow.showReadButton(cdpId);
			document.getElementById("doc-viewer-azd").contentWindow.enableCoverRead(cdpId);
		} else {
			// alert('Book installation failed. Please re-download the book.');
			AME.showMessage('Book installation failed. Please re-download the book.', 'warning', true, null);
			self.deleteBook(cdpId);
			document.getElementById("doc-viewer-azd").contentWindow.hideInstallButton(cdpId);
			document.getElementById("doc-viewer-azd").contentWindow.showDownloadButton(cdpId);
			document.getElementById("doc-viewer-azd").contentWindow.enableDownloadButton(cdpId);
			document.getElementById("doc-viewer-azd").contentWindow.disableCoverRead(cdpId);
		}
	}
	
	/**
	 * Writes the book status to database
	 * @params cdpId Id of the book
	 * @params statusValue Current status of the book expired|downloaded|installed
	 */
	var writeBookStatus = function(cdpId, statusValue) {
		var self = this;

		var fileObj = getFileObject(AME.getCdpDbFilePath());
		var table_name = 'RAF_Master';
		var values = ['raf_value'];
		var conditions = new Array(['field_id', cdpId, 'TEXT'],
								   ['raf_key', 'status', 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		while(statement.executeStep()) {
			var raf_value = statement.row.raf_value;
		}
		values = new Array(['field_id', cdpId, 'TEXT'],
						   ['raf_key', 'status', 'TEXT'],
						   ['raf_value', statusValue, 'TEXT'],
						   ['type', 'CDP', 'TEXT']);

		//	Store value for first time.
		if( typeof raf_value == 'undefined') {
			insertRowIntoDatabase(fileObj, values, table_name)
		}

		//	If value exists then update it.
		else {
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
		}
	};
	
	var writeBookExpiry = function(cdpId, expiryDate) {
		var self = this;

		var fileObj = getFileObject(AME.getCdpDbFilePath());
		var table_name = 'RAF_Master';
		var values = ['raf_value'];
		var conditions = new Array(['field_id', cdpId, 'TEXT'],
								   ['raf_key', 'expiry_date', 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		while (statement.executeStep()){
			var raf_value = statement.row.raf_value;
		}
		var values = new Array(['field_id', cdpId, 'TEXT'],
							   ['raf_key', 'expiry_date', 'TEXT'],
							   ['raf_value', expiryDate, 'TEXT'],
							   ['type', 'CDP', 'TEXT']);
							   
		//	Store value for first time.
		if (typeof raf_value == 'undefined'){
			insertRowIntoDatabase(fileObj, values, table_name)
		}
		
		//	If value exists then update it.
		else{
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
		}
	}
	
	var deleteBook = function(cdpId) {
		var self = this;

		if(confirm('Are you sure you want to remove this book?')){
			var searchDbPath = getFileObject(AME.getSearchDbFilePath()).path;
			var cdpDbPath = getFileObject(AME.getCdpDbFilePath()).path;
			var params = {
				inn : {
					searchDbPath : searchDbPath,
					cdpDbPath : cdpDbPath,
					cdpId : cdpId,
					type : 'book_remove'
				},
				out : null
			};
			window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=250, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();
			if(params.out) {
				document.getElementById("doc-viewer-azd").contentWindow.hideInstallButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.hideRemoveButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.hideReadButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.disableCoverRead(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.showDownloadButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.enableDownloadButton(cdpId);
			}
		}
	}
	
	/**
	 * Remove all books from database.
	 * @params catalogId Id of the catalog
	 */
	var removeAllBooks = function(catalogId) {
		var self = this;

		var searchDbPath = getFileObject(AME.getSearchDbFilePath()).path;
		var cdpDbPath = getFileObject(AME.getCdpDbFilePath()).path;
		var params = {
			inn : {
				searchDbPath : searchDbPath,
				cdpDbPath : cdpDbPath,
				catalogId : catalogId,
				cdpRef : config.cdpRef,
				type : 'book_remove_all'
			},
			out : null
		};
		window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=250, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();
	};
	
	/**
	 * Check and set the book expiry
	 */
	var setBookExpiry = function() {
		var self = this;
		
		//Get the list of expired books from database.
		var expiredBooks = self.getExpiredBooks();
		for(var i = 0; i < expiredBooks.length; i++) {
			//Write the book status as expired in database.
			self.writeBookStatus(expiredBooks[i], 'expired');
		}
	};
	
	/**
	 * Gets the list of expired books from database.
	 * @returns {array} List of id of expired books.
	 */
	var getExpiredBooks = function() {
		var self = this;
		
		//Get the reader token.
		var readerToken = AME.getReaderToken();
		var books = new Array();
		var expiredBooks = new Array();
		//Get the expiry info of all the books.
		var fileObj = getFileObject(AME.getCdpDbFilePath());
		var table_name = 'RAF_Master';

		var values = ['field_id', 'raf_value'];
		var conditions = new Array(['raf_key', 'expiry_date', 'TEXT'],
								   ['type', 'CDP', 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		while(statement.executeStep()) {
			var data = new Array();
			data.field_id = statement.row.field_id;
			try{
				//Decrypt the expiry date
				data.raf_value = GibberishAES.dec(statement.row.raf_value, readerToken+data.field_id);
			}
			catch(e){
				//If the decryption fails, expire the book.
				data.raf_value = '0000000000000000'
			}
			
			books.push(data);
		}
		
		//Filter expired books from list of all books.
		for(var i = 0; i < books.length; i++) {
			//check if book is expired.
			if(self.checkExpiry(books[i].raf_value)) {
				expiredBooks.push(books[i].field_id);
			}
		}
		config.expiredBooks = expiredBooks;

		return expiredBooks;
	};
	
	/**
	 * Check if the expriy date is over
	 * @params expiryDate Expiry date of the book.
	 * @returns {boolean} true if book is expired, false otherwise.
	 */
	var checkExpiry = function(expiryDate) {
		var self = this;

		var eD = new Date();
		var eMonth =  expiryDate.substr(4, 2) - 1;
		eD.setUTCFullYear(expiryDate.substr(0, 4), eMonth, expiryDate.substr(6, 2));
		eD.setUTCHours(expiryDate.substr(8, 2), expiryDate.substr(10, 2), expiryDate.substr(12, 2), expiryDate.substr(14, 3));
		var tD = new Date();
		if(tD > eD) {
			return true;
		} else {
			return false;
		}
	};
	
	var renewBook = function(cdpId,  renew_cdp_url, renew_cdp_website, renew_cdp_msg) {
		var self = this;
		
		// If catalog is expired then do not show book renewal dialog
		if(config.expired){
			// alert('Can not renew the book, as the catalog has expired.');
			AME.showMessage('Cannot renew the book, as the catalog has expired.', 'critical', true, null);
			return;
		}
		var readerToken = AME.getReaderToken();

		var params = {
			inn : {
				type : 'renew_book',
				cdpId : cdpId,
				readerToken : readerToken,
				renewWebsite: renew_cdp_website,
				renewMsg : renew_cdp_msg,
				renewCdpUrl : renew_cdp_url
			},
			out : {
				out : null,
				cdpStatus : null
			}
		};
		window.openDialog("chrome://azardi2/content/azd_book_renew.xul", "Subscription Expired", "chrome,width=600,height=260, dialog, modal, resizable=no, dependent, centerscreen", params).focus();

		if(params.out.out == false) {
			// alert('Could not renew the book.\nTry again later.');
			AME.showMessage('Could not renew the book. Try again later.', 'warning', true, null);
		} 
		else {
			//	Set the book status in catalog.
			if(params.out.cdpStatus == 'installed') {
				document.getElementById("doc-viewer-azd").contentWindow.hideInstallButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.hideExpireButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.showRemoveButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.showReadButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.enableCoverRead(cdpId);
			} else if(params.out.cdpStatus == 'default') {
				document.getElementById("doc-viewer-azd").contentWindow.hideInstallButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.hideExpireButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.hideRemoveButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.hideReadButton(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.disableCoverRead(cdpId);
				document.getElementById("doc-viewer-azd").contentWindow.showDownloadButton(cdpId);
			}

			// Remove the book from expiredBooks array so it shows in search results.
			var index = Catalog.config.expiredBooks.indexOf(cdpId);
			Catalog.config.expiredBooks.splice(index, 1);
		}
	}
	
	/**
	 * Gets the list of all the CDP associated with given catalog.
	 * @params catalogId Id of the catalog.
	 */
	var getAllCdp = function(catalogId) {
		var self = this;

		var fileObj = getFileObject(AME.getCdpDbFilePath());
		var table_name = "CAT_info"
		var conditions = new Array(['cat_id', catalogId, 'TEXT'], ['key', 'cdp_reference', 'TEXT']);
		var values = ['value'];
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		if(!statement) {
			return [];
		}
		var cdpArr;
		while(statement.executeStep()) {
			cdpArr = statement.row.value;
		}
		statement.finalize();

		config.cdpRef = cdpArr.split(',');
	};
	
	return {
		config : config,
		load : load,
		unload: unload,
		getHtmlData : getHtmlData,
		getCssData : getCssData,
		getJsData : getJsData,
		setHtmlData : setHtmlData,
		setCssData : setCssData,
		setJsData : setJsData,
		installBook : installBook,
		getButtonStatus : getButtonStatus,
		setButtonStatus : setButtonStatus,
		writeBookStatus : writeBookStatus,
		writeBookExpiry : writeBookExpiry,
		deleteBook : deleteBook,
		setBookExpiry : setBookExpiry,
		getExpiredBooks : getExpiredBooks,
		checkExpiry : checkExpiry,
		renewBook : renewBook,
		isCatalogExpired : isCatalogExpired,
		isCatalogAlreadyExpired : isCatalogAlreadyExpired,
		getCatalogExpiryDate : getCatalogExpiryDate,
		isCatalogExpiring : isCatalogExpiring,
		showCatalogRenewalDialog : showCatalogRenewalDialog,
		writeCatStatus : writeCatStatus,
		setCatalogExpiry : setCatalogExpiry,
		removeAllBooks : removeAllBooks,
		getAllCdp : getAllCdp,
		expireAllBooks : expireAllBooks,
		expiryDaysRemaining: expiryDaysRemaining,
		getPreferences: getPreferences
	}
}();
